local L = AceLibrary("AceLocale-2.2"):new("NinjutFu")
local crayon = AceLibrary("Crayon-2.0")

NinjutFu = AceLibrary("AceAddon-2.0"):new("FuBarPlugin-2.0", "AceDB-2.0", "AceEvent-2.0", "Sink-1.0")
NinjutFu.hasIcon = "Interface\\Icons\\Ability_Vanish"

NinjutFu:RegisterDB("NinjutFuDB")
NinjutFu:RegisterDefaults("profile", {
	threshCount = 20,
	--Remove in 2.3
	blindCount = 20,
	---------------
	teaCount = 10,
	buyFlash = true,
	showFlash = true,
	--Remove in 2.3
	showBlind = true,
	---------------
	showTea = true,
	compactMode = false,
	showTipTitles = true,
	tooltipSkipCategories = "",
	highend = false,
	items = "all",
	warnPoisonExpire = true,
	warnPlaySound = true,
	warnSound = [[Sound\Spells\FluteRun.wav]],
})

-- A long string with itemids of all supported items sorted into categories in the format
-- ";Category1:1234:1235;Category2:2345:2346:2347;"
--[[
local items = ";Reagents:5140:5530;" -- Flash and Blinding Powder
	.. "Energy:7676:27553;" -- Thistle Tea and Crimson Steer
	.. "Poisons"
	.. ":21835" -- Anastethic
	.. ":3775:3776" -- Crippling
	.. ":2892:2893:8984:8985:20844:22053:22054" -- Deadly Poison
	.. ":6947:6949:6950:8926:8927:8928:21927" -- Instant Poison
	.. ":5237:6951:9186" -- Mind-numbing
	.. ":10918:10920:10921:10922:22055" -- Wound Poison
	.. ";"
	.. "Stones"
	-- Rough Sharpening Stone, Rough Weightstone, Coarse Sharpening Stone, Coarse Weightstone, Heavy Sharpening Stone, Heavy Weightstone, 
	-- Solid Sharpening Stone, Solid Weightstone, Dense Sharpening Stone, Dense Weightstone, Consecrated Sharpening Stone, 
	-- Elemental Sharpening Stone, Fel Sharpening Stone, Fel Weightstone, Adamantite Sharpening Stone, Adamantite Weightstone
	-- in that order...
	.. ":2862:3239:2863:3240:2871:3241:7964:7965:12404:12643:23122:18262:23528:28420:23529:28421;"
]]--
-- Same string as commented above. Better to assign it in one row.
local items = ";Reagents:5140:5530;Energy:7676:27553;Poisons:21835:3775:3776:2892:2893:8984:8985:20844:22053:22054:6947:6949:6950:8926:8927:8928:21927:5237:6951:9186:10918:10920:10921:10922:22055;Stones:2862:3239:2863:3240:2871:3241:7964:7965:12404:12643:23122:18262:23528:28420:23529:28421;"
-- Use after 2.3:
--local items = ";Reagents:5140;Energy:7676:27553;Poisons:21835:3775:3776:2892:2893:8984:8985:20844:22053:22054:6947:6949:6950:8926:8927:8928:21927:5237:6951:9186:10918:10920:10921:10922:22055;Stones:2862:3239:2863:3240:2871:3241:7964:7965:12404:12643:23122:18262:23528:28420:23529:28421;"

-- A few functions using the string items above

-- Returns the category for a given itemid
-- (not used anywhere atm, just wrote it while I had luas magic patterns fresh in the head :P)
function NinjutFu:GetCategoryForItem(itemid)
	local found, _, category = string.find(items, ";([^:]+)[:%d]+:"..itemid.."[:;]")
	return found ~= nil and category or nil
end

-- To iterate all categories of items:
-- for category in NinjutFu:IterItemCategories() do
function NinjutFu:IterItemCategories()
	local pos = 0
	return function()
		local category
		pos, _, category = string.find(items, ";([^:]+):", pos+1)
		return pos ~= nil and category or nil
	end
end

-- To iterate all itemids in items:
-- for itemid in NinjutFu:IterAllItems() do
function NinjutFu:IterAllItems()
	local pos = 0
	return function()
		local item
		pos, _, item = string.find(items, ":(%d+)[:;]", pos+1)
		return pos ~= nil and tonumber(item) or nil
	end
end

-- To iterate all itemids in for example the category "Poisons":
-- for itemid in NinjutFu:IterCategoryItems(L["Poisons"]) do
function NinjutFu:IterCategoryItems(category)
	local pos = 0
	local items = select(3, string.find(items, ";"..category.."(:[%d:]+;)"))
	return function()
		local item
		pos, _, item = string.find(items, ":(%d+)[:;]", pos+1)
		return pos ~= nil and tonumber(item) or nil
	end
end

function NinjutFu:PopulateItemCounts()
	local set = self.db.profile.items

	self.itemCounts = {}
	if set == "all" then
		for itemid in self:IterAllItems() do
			self.itemCounts[itemid] = 0
		end
		return true
	elseif set == "none" then
		return true
	elseif set == "he60" then
		set = "5140 5530 7676 3775 3776 20844 8928 9186 10922 12404 12643 23122 18262"
--2.3		set = "5140 7676 3775 3776 20844 8928 9186 10922 12404 12643 23122 18262"
	elseif set == "he70" then
		set = "5140 5530 7676 27553 21835 3775 3776 22054 21927 9186 22055 23528 28420 23529 28421"
--2.3		set = "5140 7676 27553 21835 3775 3776 22054 21927 9186 22055 23528 28420 23529 28421"
	end

	local itemid
	local pos = 0
	while true do
		pos, itemid = select(2, string.find(set, "^ *([^%s]+) *", pos + 1))
		if not itemid then
			break
		end
		self.itemCounts[tonumber(itemid)] = 0
	end
end

function NinjutFu:OnInitialize()
end

function NinjutFu:OnEnable()
	self:PopulateItemCounts()

	if self.poisonSpell == nil then
		self:RegisterEvent("PLAYER_LOGIN", "GetPoisonSpell")
	end
	self:RegisterBucketEvent("BAG_UPDATE", 0.5, "Update")
	self:RegisterEvent("MERCHANT_SHOW", "RefillFlash")
	self:PoisonWatchChanged()
end

function NinjutFu:OnDisable()
	self.itemCounts = nil
end

function NinjutFu:GetPoisonSpell()
	if GetLocale() == "enUS" then
		self.poisonSpell = "Poisons"
		return true
	else
		-- Search the spellbook for the Poisons-button
		local numSpells = select(4, GetSpellTabInfo(1)) -- Only need to search the General-tab
		for i = 1, numSpells do
			local texture = GetSpellTexture(i, BOOKTYPE_SPELL)
			if string.find(texture, "Trade_BrewPoison") then
				self.poisonSpell = GetSpellName(i, BOOKTYPE_SPELL) -- Get the localized spellname
				self:UnregisterEvent("PLAYER_LOGIN")
				return true
			end
		end
	end
	return nil
end

function NinjutFu:OnDataUpdate()
	self:CountItems()
end

function NinjutFu:OnTextUpdate()
	local barText = self:GetTitle()
	local p = self.db.profile

	local notFirst = false
	local seperator = p.compactMode and "/" or " "

	if p.showFlash then
		barText = self:Colorize(self.itemCounts[5140], p.threshCount)
		notFirst = true
		if not p.compactMode then
			barText = barText .. "F"
		end
	end
	-- Remove in 2.3
	if p.showBlind then
		if notFirst then
			barText = barText .. seperator .. self:Colorize(self.itemCounts[5530], 20)
		else
			barText = self:Colorize(self.itemCounts[5530], 20)
			notFirst = true
		end
		if not p.compactMode then
			barText = barText .. "B"
		end
	end
	---------------
	if p.showTea then
		if notFirst then
			barText = barText .. seperator .. self:Colorize(self.itemCounts[7676], 10)
		else
			barText = self:Colorize(self.itemCounts[7676], 10)
		end
		if not p.compactMode then
			barText = barText .. "T"
		end
	end

	self:SetText(barText)
end

function NinjutFu:Colorize(count, stacksize)
	return crayon:Colorize(crayon:GetThresholdHexColor(count, stacksize/4, stacksize/2, stacksize*0.65, stacksize*0.8, stacksize*0.95), count)
end

local function GetItemID(bag, slot)
	local linktext = GetContainerItemLink(bag, slot)
	if linktext then
		local itemid = select(3, string.find(linktext, "item:(%d+):"))
		return tonumber(itemid)
	else
		return nil
	end
end

function NinjutFu:CountItems()
	local itemid, count
	local itemCounts = self.itemCounts

	-- Reset all counts to zero
	for itemid in pairs(itemCounts) do
		itemCounts[itemid] = 0
	end

	-- Iterate all bagslots
	for i = NUM_BAG_FRAMES, 0, -1 do
		for j = GetContainerNumSlots(i), 1, -1 do
			itemid = GetItemID(i, j)
			if (itemCounts[itemid]) then
				_, count = GetContainerItemInfo(i, j)
				itemCounts[itemid] = itemCounts[itemid] + count
			end
		end
	end
end

function NinjutFu:OnClick()
	if self.poisonSpell == nil then
		-- If the spell wasn't found at login, try again on click
		if self:GetPoisonSpell() then CastSpellByName(self.poisonSpell) end
	else
		self.GetPoisonSpell = nil --Don't need it anymore :P
		CastSpellByName(self.poisonSpell)
	end
end

function NinjutFu:RefillFlash()
	local class = select(2, UnitClass("player"))
	if (class == "ROGUE" and self.db.profile.buyFlash) then
		local numItems = GetMerchantNumItems()
		local currentCount = self.itemCounts[5140]
		local diff = self.db.profile.threshCount - currentCount
		if (diff > 0) then
			local link, itemid
			for i=0, numItems do
				link = GetMerchantItemLink(i)
				if (link) then
					_,_,itemid = string.find(link, "item:(%d+):")
					if tonumber(itemid) == 5140 then
						while (diff > 20) do
							BuyMerchantItem(i, 20)
							diff = diff - 20
						end
						BuyMerchantItem(i, diff)
					end
				end
			end
		end
	end
end


function NinjutFu:PoisonWatchChanged()
	if self.db.profile.warnPoisonExpire then
		self:RegisterEvent("SpecialEvents_PlayerItemBuffLost", "ItemBuffLost")
		self:RegisterEvent("SpecialEvents_PlayerItemBuffGained", "ItemBuffGained")
	else
		self:UnregisterEvent("SpecialEvents_PlayerItemBuffLost")
		self:UnregisterEvent("SpecialEvents_PlayerItemBuffGained")
	end
end

local shortMHBuff = false
local shortOHBuff = false
function NinjutFu:ItemBuffLost(name, rank, mh)
	if mh then
		if shortMHBuff then
			shortMHBuff = false
			return
		end
	else
		if shortOHBuff then
			shortOHBuff = false
			return
		end
	end
	self:Pour(L["%s poison faded"]:format(L[mh and "MH" or "OH"]), 1, 0, 0)
	if self.db.profile.warnPlaySound then
		PlaySoundFile(self.db.profile.warnSound)
	end
--	DEFAULT_CHAT_FRAME:AddMessage(string.format("|cffffff78NinjutFu:|r " .. "%s lost on %s", name, L[mh and "MH" or "OH"]), 1, 0, 0, nil, 5)
end

function NinjutFu:ItemBuffGained(name, rank, mh)
	local timeleft = select(mh and 2 or 5, GetWeaponEnchantInfo())
--	DEFAULT_CHAT_FRAME:AddMessage(string.format("|cffffff78NinjutFu:|r " .. "%s gained on %s", name, L[mh and "MH" or "OH"]), 1, 0, 0, nil, 5)
	if timeleft <= 10000 then
		if mh then
			shortMHBuff = true
		else
			shortOHBuff = true
		end
	end
end
